from vertibit_osi_image_generator.shared_aux.file_scanner_shared import scan_directory
from vertibit_osi_image_generator.shared_aux.language_package_managers import get_language_package_managers


# Main function for scanning directory
def main_language_scanner(directory: str):
    package_manager_files = get_language_package_managers()
    for language_info in package_manager_files:
        package_manager_file = language_info["package_manager_file"]['file_name']
        package_manager_extension = language_info["package_manager_file"]['extension']
        matching_file = scan_directory(
            directory, (package_manager_file, package_manager_extension))
        if matching_file != None:
            language_info['package_dir_path'] = matching_file
            return language_info
        else:
            print("%s package manager not found" % (language_info["language"]))



from vertibit_osi_image_generator.verti_file_config_aux.file_config import load_config_schema, load_verti_config, apply_defaults, validate_config


def load_and_validate_config(image_config_path='verti-osi.yaml', config_file='config.json'):
    config_schema_json = load_config_schema(config_file=config_file)
    provided_config = load_verti_config(config_path=image_config_path)

    config_with_defaults = apply_defaults(
        config=provided_config, schema=config_schema_json)
    config = validate_config(
        config=config_with_defaults, schema=config_schema_json)

    return config



def retrieve_language_base_images():
    return [
        {
            "language": "Python",
            "images": {
                "build": "cgr.dev/chainguard/python:latest-dev",
                "run-time": "cgr.dev/chainguard/python:latest"
            },
            "base-and-runtime": {
                "cgr": {
                    "build": "cgr.dev/chainguard/python:latest-dev",
                    "run-time": "cgr.dev/chainguard/python:latest"
                },
                "normal": {
                    "build": "python:latest",
                    "run-time": "python:slim"
                }
            }
        },
        {
            "language": "NodeJs",
            "images": {
                "build": "cgr.dev/chainguard/node:latest-dev",
                "run-time": "cgr.dev/chainguard/node:latest"
            },
            "base-and-runtime": {
                "cgr": {
                    "build": "cgr.dev/chainguard/node:latest-dev",
                    "run-time": "cgr.dev/chainguard/node:latest"
                },
                "normal": {
                    "build": "node:lts",
                    "run-time": "node:lts-slim"
                }
            }
        },
        {
            "language": "Maven",
            "images": {
                "build": "cgr.dev/chainguard/maven:latest-dev",
                "run-time": "cgr.dev/chainguard/maven:latest"
            },
            "base-and-runtime": {
                "cgr": {
                    "build": "cgr.dev/chainguard/maven:latest-dev",
                    "run-time": "cgr.dev/chainguard/maven:latest"
                },
                "normal": {
                    "build": "",
                    "run-time": ""
                }
            }
        },
        {
            "language": "Gradle",
            "images": {
                "build": "",
                "run-time": "cgr.dev/chainguard/gradle:latest"
            },
            "base-and-runtime": {
                "cgr": {
                    "build": "",
                    "run-time": "cgr.dev/chainguard/gradle:latest"
                },
                "normal": {
                    "build": "",
                    "run-time": ""
                }
            }
        },
        {
            "language": "dotnet",
            "images": {
                "build": "",
                "run-time": ""
            },
            "base-and-runtime": {
                "cgr": {
                    "build": "",
                    "run-time": ""
                },
                "normal": {
                    "build": "",
                    "run-time": ""
                }
            }
        },
    ]


def extract_images(language: str):
    language_base_images = retrieve_language_base_images()
    images = get_images_by_language(language_base_images, language)

    return images


def get_images_by_language(elements, language, image_type: str = 'normal'):
    """
    Extracts the 'images' object for a specific language from a list of elements.

    Parameters:
        elements (list): List of dictionaries containing 'language' and 'images'.
        language (str): The language to filter by.

    Returns:
        dict: The 'images' object for the matching language, or None if not found.
    """
    for element in elements:
        if element.get("language").lower() == language.lower():
            if image_type.lower() == 'cgr':
                return element.get("base-and-runtime", {}).get("cgr", {})
            else:
                return element.get("base-and-runtime", {}).get("normal", {})
    return None


def get_language_package_managers():
    return [
        {
            "language": "Python",
            "package_manager_file": {
                "file_name": "requirements",
                "extension": "txt"
            },
            "main_modules": {
                "filenames": ['app', 'server', 'main', 'manage'],
                "extension": 'py'
            },
            "start_scripts": [],
            "lock_files": [],
        },
        {

            "language": "NodeJs",
            "package_manager_file": {
                "file_name": "package",
                "extension": "json"
            },
            "main_modules": {
                "filenames": ['app', 'server', 'index', 'main'],
                "extension": 'js'
            },
            "start_scripts": ["start", "start:prod"],
            "lock_files": [
                {"filename": 'package-lock', "extension": 'json',
                    "package_manager": "npm"},
                {"filename": 'yarn', "extension": 'lock', "package_manager": "yarn"},
            ]
        },
        {

            "language": "Maven",
            "package_manager_file": {
                "file_name": "",
                "extension": ""
            },
            "main_modules": {
            },
            "start_scripts": [],
            "lock_files": [],
        },
        {

            "language": "Gradle",
            "package_manager_file": {
                "file_name": "",
                "extension": ""
            },
            "main_modules": {
            },
            "start_scripts": [],
            "lock_files": [],
        }
    ]



from vertibit_osi_image_generator.shared_aux.file_scanner_shared import scan_for_json_key_value, open_json_file, find_matching_key, scan_directory, scan_for_main_modules


def generate_nodejs_start_and_install_commands(root_directory, source_directory, language_info):
    start_command = ''
    package_manager_info = scan_for_js_package_manager_to_use(
        root_directory, language_info)
    installation_command = nodejs_installation_command(package_manager_info)
    start_script_key, start_script_key_exists = scan_nodejs_start_command(
        language_info['package_dir_path'], language_info)

    if start_script_key_exists:
        start_command = generate_nodejs_start_command(
            package_manager_info['package_manager'], start_script_key)
    else:
        main_module_path, main_module_path_exists = scan_for_main_modules(
            language_info['main_modules'], root_directory, source_directory)

        if main_module_path_exists:
            start_command = generate_standard_nodejs_start_command(
                main_module_path)
        else:
            # come back to throw error
            pass

    return {'start_command': start_command, 'installation_command': installation_command}


def scan_nodejs_start_command(file_path: str, language_info):
    scripts_object = retrieve_nodejs_scripts(file_path)

    start_key = find_matching_key(
        scripts_object, language_info['start_scripts'])

    if start_key != None:
        return (start_key, True)

    return (None, False)


def scan_nodejs_version(file_path: str):
    json_content = open_json_file(file_path)
    node_version = scan_for_json_key_value(json_content, path="engines.node")

    if node_version != None:
        return (node_version, True)

    return (None, False)


def retrieve_nodejs_scripts(file_path: str):
    json_content = open_json_file(file_path)
    scripts_object = scan_for_json_key_value(json_content, path="scripts")

    if scripts_object:
        return scripts_object

    return {}


def generate_nodejs_start_command(package_manager, start_command):
    if package_manager == 'yarn':
        return f'["{package_manager}", "{start_command}"]'

    if package_manager == 'npm':
        return f'["{package_manager}", "run", "{start_command}"]'


def generate_standard_nodejs_start_command(entry_file):
    return f'["node", "{entry_file}"]'


def scan_for_js_package_manager_to_use(root_directory, language_info):
    for lock_file in language_info['lock_files']:
        file_name = lock_file['filename']
        file_path = scan_directory(
            root_directory, (lock_file['filename'], lock_file['extension']))
        if file_path != None:
            print(f"lock file identified: {file_name}")
            print(f"package manager to be used: {
                  lock_file['package_manager']}")
            return {'package_manager': lock_file['package_manager'], 'locked': True, 'lock_file_path': file_path}

    return {'package_manager': "npm", 'locked': False}


def nodejs_installation_command(package_info):
    if package_info['package_manager'] == 'npm':
        if package_info['locked']:
            return f'npm ci'
        else:
            return f'npm install'
    elif package_info['package_manager'] == 'yarn':
        if package_info['locked']:
            return f'yarn install --frozen-lockfile'
        else:
            return f'yarn install'


def generate_js_package_config_requirements(package_info, package_manager_dir):
    if package_info['locked']:
        return f"{package_manager_dir} {package_info['lock_file_path']}"
    else:
        return package_manager_dir



import os
import json
from typing import Dict, Any, Tuple, Union, List


def scan_for_json_key_value(data: Dict[Any, Any], path: Union[str, List[str]], separator: str = '.') -> Any:
    """
    Retrieves a value from a nested dictionary using a path-like string or list of keys.

    Parameters:
        data (Dict[Any, Any]): The dictionary to search through
        path (Union[str, List[str]]): Path to the desired value. Can be either:
            - A string with keys separated by separator (e.g., "key1.key2.key3")
            - A list of keys (e.g., ["key1", "key2", "key3"])
        separator (str): The separator to use when path is a string (default: '.')

    Returns:
        Any: The value found at the specified path

    Raises:
        KeyError: If the path cannot be found in the dictionary
        ValueError: If the path is empty or invalid
        TypeError: If the data is not a dictionary
    """
    if not isinstance(data, dict):
        raise TypeError("Input data must be a dictionary")

    if not path:
        raise ValueError("Path cannot be empty")

    # Convert string path to list of keys
    keys = path.split(separator) if isinstance(path, str) else path

    current = data
    for key in keys:
        current = current[key]
        if current:
            print(f"Key : {key}, identified")
            return current
        else:
            print(f"Key : {key}, not found")
            pass
    return {}


def scan_directory(directory: str, file_spec: Tuple[str, str]) -> str | None:
    """
    Scans a provided directory for a file matching specific filename and extension.
    Only checks the provided directory, not subdirectories.

    Parameters:
        directory (str): The directory to scan.
        file_spec (Tuple[str, str]): Tuple containing (filename, extension) pair
                                    Example: ('package', 'json')
                                    Note: Extension should not include the dot.

    Returns:
        str | None: Full path of the matching file if found, None otherwise.

    Raises:
        ValueError: If the provided directory is invalid or file_spec format is incorrect.
    """
    if not os.path.isdir(directory):
        raise ValueError(
            f"The provided path is not a valid directory: {directory}")

    # Validate file_spec format
    filename, extension = file_spec
    if not isinstance(filename, str) or not isinstance(extension, str):
        raise ValueError(
            "file_spec must be a tuple of (filename, extension) as strings")
    if not filename or not extension:
        raise ValueError(
            "Both filename and extension must be non-empty strings")
    if '.' in filename or '.' in extension:
        raise ValueError(
            "Filename and extension should be provided separately without dots")

    # List files in the directory (not recursive)
    for file in os.listdir(directory):
        file_path = os.path.join(directory, file)

        # Skip if not a file
        if not os.path.isfile(file_path):
            continue

        # Split the filename and extension
        name, ext = os.path.splitext(file)
        # Remove the dot from extension
        ext = ext[1:] if ext.startswith('.') else ext

        # Check if the file matches the specification
        if name.lower() == filename.lower() and ext.lower() == extension.lower():
            return file_path

    return None

def open_json_file(file_path: str) -> Dict[Any, Any]:
    """
    Reads a JSON file and returns its contents as a dictionary.

    Parameters:
        file_path (str): Path to the JSON file to read.

    Returns:
        Dict[Any, Any]: Dictionary containing the JSON file contents.

    Raises:
        FileNotFoundError: If the specified file does not exist.
        json.JSONDecodeError: If the file contains invalid JSON.
        PermissionError: If the program lacks permission to read the file.
    """
    try:
        with open(file_path, 'r', encoding='utf-8') as file:
            return json.load(file)
    except FileNotFoundError:
        raise FileNotFoundError(f"The file {file_path} was not found")
    except json.JSONDecodeError as e:
        raise json.JSONDecodeError(f"Invalid JSON format in {
                                   file_path}: {str(e)}", e.doc, e.pos)
    except PermissionError:
        raise PermissionError(
            f"Permission denied when trying to read {file_path}")
    except Exception as e:
        raise Exception(f"An error occurred while reading {
                        file_path}: {str(e)}")


def read_text_file(file_path: str) -> str:
    """
    Reads a text file and returns its contents as a string.

    Parameters:
        file_path (str): Path to the text file to read.

    Returns:
        str: Contents of the text file.

    Raises:
        FileNotFoundError: If the specified file does not exist.
        PermissionError: If the program lacks permission to read the file.
    """
    try:
        with open(file_path, 'r', encoding='utf-8') as file:
            return file.read()
    except FileNotFoundError:
        raise FileNotFoundError(f"The file {file_path} was not found")
    except PermissionError:
        raise PermissionError(
            f"Permission denied when trying to read {file_path}")
    except Exception as e:
        raise Exception(f"An error occurred while reading {
                        file_path}: {str(e)}")


def scan_txt_file_for_key(text: str, search_key: str) -> Tuple[str | None, bool]:
    """
    Scans a string for a specific search key and returns the complete word containing that key
    along with a boolean indicating if the key was found.

    Parameters:
        text (str): The text to search through
        search_key (str): The key to search for (case-insensitive)

    Returns:
        Tuple[str | None, bool]: A tuple containing:
            - The complete word containing the search key (or None if not found)
            - Boolean indicating if the search key was found

    Example:
        text = "Hello World package.json is here"
        word, found = find_word_with_key(text, "package")
        # Returns: ("package.json", True)
    """
    if not text or not search_key:
        return None, False

    # Convert to lowercase for case-insensitive search
    text_lower = text.lower()
    search_key_lower = search_key.lower()

    # Split text into words
    words = text.split()

    # Search for the key in each word
    for word in words:
        if search_key_lower in word.lower():
            return word, True

    return None, False


def find_matching_key(obj, keys_to_check):
    """
    Check if an object (dictionary) contains any of the specified keys.

    Args:
        obj (dict): The dictionary to check.
        keys_to_check (list): A list of keys to search for.

    Returns:
        str: The name of the first matching key, or None if no keys match.
    """
    for key in keys_to_check:
        if key in obj:
            return key
    return None

def scan_for_main_modules(main_modules_info, root_directory, source_directory):
    for file_name in main_modules_info['filenames']:
        file_exists_in_root_dir = scan_directory(
            root_directory, (file_name, main_modules_info['extension']))
        if file_exists_in_root_dir != None:
            return (file_name, True)

        file_exists_in_source_dir = scan_directory(
            source_directory, (file_name, main_modules_info['extension']))
        if file_exists_in_source_dir != None:
            return (file_name, True)

    return (None, False)

def clean_module_path(path: str) -> str:
    # Remove leading dots
    cleaned = path.lstrip('.')
    
    # Remove .py extension if it exists
    if cleaned.endswith('.py'):
        cleaned = cleaned[:-3]
        
    return cleaned


import os
import random
import string
import shutil


def delete_files(file_paths):
    """
    Deletes multiple files at the specified file paths.

    Args:
        file_paths (list): A list of file paths to be deleted.

    Returns:
        dict: A dictionary with file paths as keys and results as values.
    """
    results = {}

    for file_path in file_paths:
        try:
            os.remove(file_path)
            results[file_path] = "Deleted successfully."
        except FileNotFoundError:
            results[file_path] = "File does not exist."
        except PermissionError:
            results[file_path] = "Permission denied."
        except Exception as e:
            results[file_path] = f"Error: {e}"

    for file, result in results.items():
        print(f"{file}: {result}")

    return results


def generate_random_string(length=4):
    return ''.join(random.choices(string.ascii_lowercase + string.digits + string.octdigits, k=length))

def format_directory(directory:str) -> str:
    """
    Format directory string according to specified rules:
    - Verifies input is a string
    - If directory is ".", convert to "./"
    - If directory is anything else, add "/" to produce "/directory"
    - If directory already follows the standard format, leave it unchanged
    
    Args:
        directory (str): Directory string to format
    
    Returns:
        str: Properly formatted directory string
        
    Raises:
        TypeError: If directory is not a string
    """
    # Check if input is a string
    if not isinstance(directory, str):
        raise TypeError("Directory must be a string")
    
    if not directory:
        return "/"
    
    # Case: directory is "."
    if directory == ".":
        return "./"
    
    # Case: directory already starts with "/"
    if directory.startswith("/"):
        return directory
    
    # Case: directory already ends with "/"
    if directory.endswith("/"):
        return "/" + directory
    
    # Default case: add "/" prefix to directory
    return "/" + directory


def copy_file(source_path, destination_dir):
    """
    Copy a file from source_path to destination_dir.
    
    Args:
        source_path (str): Path to the source file
        destination_dir (str): Directory where the file will be copied to
    """
    # Check if source file exists
    if not os.path.isfile(source_path):
        print(f"Error: Source file '{source_path}' does not exist.")
        return False
    
    # Create destination directory if it doesn't exist
    if not os.path.exists(destination_dir):
        print(f"Destination directory '{destination_dir}' does not exist. Creating it...")
        os.makedirs(destination_dir)
    
    # Get just the filename from the source path
    file_name = os.path.basename(source_path)
    
    # Create the full destination path
    destination_path = os.path.join(destination_dir, file_name)
    
    # Copy the file
    try:
        shutil.copy2(source_path, destination_path)
        print(f"Successfully copied '{source_path}' to '{destination_path}'")
        return True
    except Exception as e:
        print(f"Error copying file: {e}")
        return False


from vertibit_osi_image_generator.nodejs.nodejs_config import scan_for_js_package_manager_to_use, generate_nodejs_start_and_install_commands, generate_js_package_config_requirements

def prepeare_nodejs_docker_file_generation(root_directory, source_directory, language_info):
    package_manager_info = scan_for_js_package_manager_to_use(
        root_directory, language_info)
    start_and_installation_command = generate_nodejs_start_and_install_commands(
        root_directory, source_directory, language_info)
    package_directories = generate_js_package_config_requirements(
        package_manager_info, language_info['package_dir_path'])
    return {'package_directories': package_directories, 'entry_point': start_and_installation_command['start_command'], 'install_command': start_and_installation_command['installation_command'], 'source_dir': source_directory}


import typer
from queue import Queue
import subprocess
import threading
import queue


def stream_subprocess_output(process, queue, stream_name):
    """
    Read from a subprocess stream (stdout/stderr) line by line and put into a queue
    """
    stream = process.stdout if stream_name == 'stdout' else process.stderr
    for line in iter(stream.readline, ''):
        queue.put((stream_name, line.strip()))
    stream.close()


def run_and_log_commands(command):
    try:
        # Log the command being executed
        typer.echo(f"Executing command: {command}")

        # Create the subprocess
        process = subprocess.Popen(
            command,
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=True,
            bufsize=1  # Line buffered
        )

        # Create a queue to hold the output
        output_queue = Queue()

        # Create threads to read stdout and stderr
        stdout_thread = threading.Thread(
            target=stream_subprocess_output,
            args=(process, output_queue, 'stdout')
        )
        stderr_thread = threading.Thread(
            target=stream_subprocess_output,
            args=(process, output_queue, 'stderr')
        )

        # Start the threads
        stdout_thread.start()
        stderr_thread.start()

        # Print output as it becomes available
        error_output = []
        while stdout_thread.is_alive() or stderr_thread.is_alive() or not output_queue.empty():
            try:
                # Get output from queue with timeout
                stream_name, line = output_queue.get(timeout=0.1)
                if stream_name == 'stdout':
                    typer.echo(line)
                else:
                    typer.echo(line, err=True)
                    error_output.append(line)
            except queue.Empty:
                continue

        # Wait for process to complete
        process.wait()

        # Wait for reader threads to finish
        stdout_thread.join()
        stderr_thread.join()

        # Check for errors
        if process.returncode != 0:
            error_message = "\n".join(error_output)
            typer.echo(f"Error executing command {command}:\n{
                       error_message}", err=True)
            raise typer.Exit(code=process.returncode)

    except Exception as e:
        typer.echo(f"An unexpected error occurred: {e}", err=True)
        raise typer.Exit(1)



from vertibit_osi_image_generator.shared_aux.comand_runner import run_and_log_commands


def checkov_check(file_path):
    """
    Run Checkov on the specified directory.

    Args:
        directory (str): The directory to run Checkov on.
        checkov_path (str): The path to the Checkov executable.

    Returns:
        str: The output of the Checkov command.
    """
    framework = 'dockerfile'

    command = f"checkov --file {file_path} --framework {framework} --skip-check CKV_DOCKER_2"
    result = run_and_log_commands(command)
    return result



import os
from vertibit_osi_image_generator.shared_aux.file_scanner_shared import read_text_file, scan_txt_file_for_key, scan_directory, scan_for_main_modules, clean_module_path


def generate_python_start_command(root_directory, source_directory, language_info):
    start_comand = ''
    django_project, is_django_app = scan_for_django_app(
        language_info['package_dir_path'], root_directory)

    if is_django_app:
        start_comand = generate_django_start_command(django_project)
        return start_comand

    fast_api_main_module_path, is_fast_api_app = scan_for_fastapi_app(
        language_info['package_dir_path'], language_info['main_modules'], root_directory, source_directory)

    if is_fast_api_app:
        module_path = replace_slash_with_dot(fast_api_main_module_path)
        cleaned_module_path = clean_module_path(module_path)
        start_comand = generate_fastapi_start_command(cleaned_module_path)
        return start_comand

    flask_main_module_path, is_flask_app = scan_for_flask_app(
        language_info['package_dir_path'], language_info['main_modules'], root_directory, source_directory)

    if is_flask_app:
        module_path = replace_slash_with_dot(flask_main_module_path)
        cleaned_module_path = clean_module_path(module_path)
        start_comand = generate_flask_start_command(cleaned_module_path)
        return start_comand

    main_module_path, main_module_exists = scan_for_main_modules(
        main_modules_info=language_info['main_modules'], root_directory=root_directory, source_directory=source_directory)

    if main_module_exists:
        start_comand = generate_standard_python_start_command(main_module_path)
        return start_comand

    # Throw error if none


def find_django_directory(root_directory):
    # List of required files
    required_files = {"settings.py", "urls.py", "asgi.py", "wsgi.py"}

    # Walk through the directory tree
    for dirpath, _, filenames in os.walk(root_directory):
        # Check if the set of required files is a subset of the current directory's files
        if required_files.issubset(set(filenames)):
            # Return the name of the directory
            return os.path.basename(dirpath)

    return None  # Return None if no such directory is found


def scan_for_django_app(file_path: str, root_directory):
    text_file_content = read_text_file(file_path)
    _, package_exists = scan_txt_file_for_key(text_file_content, 'django')
    django_manage_file_exists = scan_directory(
        root_directory, ('manage', 'py'))

    django_directory = find_django_directory(root_directory)

    if package_exists and django_manage_file_exists != None and django_directory != None:
        return (django_directory, True)

    return (None, False)


def scan_for_fastapi_app(file_path: str, main_modules_info, root_directory, source_directory):
    text_file_content = read_text_file(file_path)
    _, package_exists = scan_txt_file_for_key(text_file_content, 'fastapi')

    if package_exists:
        for file_name in main_modules_info['filenames']:
            file_exists_in_root_dir = scan_directory(
                root_directory, (file_name, main_modules_info['extension']))
            if file_exists_in_root_dir != None:
                return (file_exists_in_root_dir, True)

            file_exists_in_source_dir = scan_directory(
                source_directory, (file_name, main_modules_info['extension']))
            if file_exists_in_source_dir != None:
                return (file_exists_in_source_dir, True)

    return (None, False)


def scan_for_flask_app(file_path: str, main_modules_info, root_directory, source_directory):
    text_file_content = read_text_file(file_path)
    _, package_exists = scan_txt_file_for_key(text_file_content, 'flask')

    if package_exists:
        for file_name in main_modules_info['filenames']:
            file_exists_in_root_dir = scan_directory(
                root_directory, (file_name, main_modules_info['extension']))
            if file_exists_in_root_dir != None:
                return (file_exists_in_root_dir, True)

            file_exists_in_source_dir = scan_directory(
                source_directory, (file_name, main_modules_info['extension']))
            if file_exists_in_source_dir != None:
                return (file_exists_in_source_dir, True)

    return (None, False)


def generate_flask_start_command(entry_file):
    start_command = f'["python","-m","gunicorn", "-w" ,"4", "-b", "0.0.0.0:8000", "{
        entry_file}:app"]'

    return start_command


def generate_fastapi_start_command(entry_file):
    start_command = f'["python","-m","uvicorn" ,"{
        entry_file}:app" ,"--host", "0.0.0.0", "--port", "8000", "--workers", "4"]'

    return start_command


def generate_django_start_command(project_dir):
    start_command = f'["python","-m","gunicorn", "-w" ,"4", "-b", "0.0.0.0:8000", "{
        project_dir}.wsgi:app"]'

    return start_command


def generate_standard_python_start_command(entry_file):
    start_command = f'["python","{entry_file}"]'

    return start_command


def replace_slash_with_dot(path):
    """
    Replaces all occurrences of '/' with '.' in the given path string.

    Parameters:
    path (str): The path string to be converted.

    Returns:
    str: The converted path with '.' instead of '/'.
    """
    return path.replace("/", ".")



from vertibit_osi_image_generator.shared_aux.comand_runner import run_and_log_commands


def run_docker_image(image_name, ports={8000: 8000}):
    """
    Extracts a .tar file, loads a Docker image, and runs it, publishing specified ports.

    Args:
        ports (dict): A dictionary of ports to publish, e.g., {80: 8080} (host:container).
    Returns:
        str: The result of the Docker run command or an error message.
    """
    try:
        # Step 3: Build the Docker run command
        port_mappings = ""
        if ports:
            port_mappings = " ".join(
                [f"-p {host}:{container}" for host, container in ports.items()])

        run_cmd = f"docker run -it {port_mappings} {image_name}"

        run_and_log_commands(run_cmd)

    except Exception as e:
        return f"An unexpected error occurred: {e}"



from typing import List, Dict, Any


def add_env_vars(env_list: List[str], env_type: str) -> List[Dict[str, str]]:
    """
    Converts a list of environment variables in the format "KEY=VALUE" into a list of dictionaries
    with the schema's required format: {"name": "KEY", "value": "VALUE", "type": "runtime/buildtime/both"}.
    """
    env_vars_formatted = []
    for env in env_list:
        if '=' in env:
            key, value = env.split('=', 1)
            env_vars_formatted.append({
                "name": key,
                "value": value,
                "type": env_type
            })
    return env_vars_formatted


def format_image_params_to_schema(
    platforms: str = "",
    repository_branch: str = "",
    repository_url: str = "",
    env_vars: List[str] = [],
    env_vars_rt: List[str] = [],
    env_vars_bt: List[str] = [],
    build_commands: List[str] = [],
    pre_build_commands: List[str] = [],
    root_directory: str = './',
    source_directory: str = './',
    image_name: str = "",
    daemon: str = 'docker',
    output: str = "",
    delete_generated_dockerfile: bool = False,
    port: int = 8080,
) -> Dict[str, Any]:

    # Initialize the schema dictionary
    schema_dict: Dict[str, Any] = {
        "image-name": image_name,
        "source-directory": source_directory,
        "root-directory": root_directory,
        "delete-generated-dockerfile": delete_generated_dockerfile,
        "daemon": daemon,
        "output-type": output if output else "normal",
        # Default platforms
        "platform": platforms.split(',') if platforms else ["linux/amd64", "linux/arm64"],
        # Empty array if none provided
        "pre-build": pre_build_commands if pre_build_commands else [],
        "build": build_commands if build_commands else [],  # Empty array if none provided
        "env-vars": [],  # Will be populated below
        "port": str(port),
    }

    # Add remote-source-repository if repository_url is provided
    if repository_url:
        schema_dict["remote-source-repository"] = {
            "git": {
                "url": repository_url,
                "branch": repository_branch if repository_branch else "main"
            }
        }

    # Add environment variables
    schema_dict["env-vars"].extend(add_env_vars(env_vars, "both"))
    schema_dict["env-vars"].extend(add_env_vars(env_vars_rt, "runtime"))
    schema_dict["env-vars"].extend(add_env_vars(env_vars_bt, "buildtime"))

    return schema_dict


def format_dockerfile_params_to_schema(
    repository_branch: str = "",
    repository_url: str = "",
    env_vars: List[str] = [],
    env_vars_rt: List[str] = [],
    env_vars_bt: List[str] = [],
    build_commands: List[str] = [],
    pre_build_commands: List[str] = [],
    root_directory: str = './',
    source_directory: str = './',
    port: int = 8080,
) -> Dict[str, Any]:

    # Initialize the schema dictionary
    schema_dict: Dict[str, Any] = {
        "source-directory": source_directory,
        "root-directory": root_directory,
        # Empty array if none provided
        "pre-build": pre_build_commands if pre_build_commands else [],
        "build": build_commands if build_commands else [],  # Empty array if none provided
        "env-vars": [],  # Will be populated below
        "port": str(port),
    }

    # Add remote-source-repository if repository_url is provided
    if repository_url:
        schema_dict["remote-source-repository"] = {
            "git": {
                "url": repository_url,
                "branch": repository_branch if repository_branch else "main"
            }
        }

    # Add environment variables
    schema_dict["env-vars"].extend(add_env_vars(env_vars, "both"))
    schema_dict["env-vars"].extend(add_env_vars(env_vars_rt, "runtime"))
    schema_dict["env-vars"].extend(add_env_vars(env_vars_bt, "buildtime"))

    return schema_dict



from vertibit_osi_image_generator.python.python_config import generate_python_start_command

def prepare_python_docker_file_generation(root_directory, source_directory, language_info):
    start_command = generate_python_start_command(
        root_directory, source_directory, language_info)
    return {'entry_point': start_command, 'source_dir': source_directory}


import os
import json
import yaml
from jsonschema import Draft7Validator
from importlib.resources import files


def load_config_schema(config_file='config.json'):
    """Load the JSON schema from a file within the package."""
    config_file = files("vertibit_osi_image_generator").joinpath(
        "config/config.json")
    if not config_file.is_file():
        raise FileNotFoundError(f"Config file not found at: {config_file}")
    with config_file.open("r") as file:
        return json.load(file)


def load_verti_config(config_path='verti-osi.yaml'):
    """
    Load and validate the configuration from the YAML file using the schema.

    Args:
        config_path (str): Path to the YAML configuration file.
        schema_path (str): Path to the JSON schema file.

    Returns:
        dict: The validated and processed configuration.
    """

    with open(config_path, "r") as file:
        config_dict = yaml.safe_load(file) or {}

    return config_dict


def apply_defaults(config, schema):
    """Recursively apply default values from schema to the config."""
    for key, value in schema.get("properties", {}).items():
        if key == "remote-source-repository" and key not in config:
            continue  # Skip applying defaults for remote-source-repository

        if key == "images" and key not in config:
            continue  # Skip applying defaults for remote-source-repository

        if key not in config and "default" in value:
            config[key] = value["default"]

        if isinstance(value.get("properties"), dict):  # If nested object, recurse
            config[key] = apply_defaults(config.get(key, {}), value)
    return config


def validate_config(config, schema):
    """
    Validate the config.

    Args:
        config (dict): The user-provided configuration.
        schema (dict): The JSON schema to validate against.

    Returns:
        dict: The validated configuration.
    """
    # Validate the config against the schema
    validator = Draft7Validator(schema)
    errors = sorted(validator.iter_errors(config), key=lambda e: e.path)

    if errors:
        for error in errors:
            print(f"Validation error: {error.message}")
        raise ValueError(f"Validation error: {error.message}")

    return config


def verify_config_file(file_dir: str, verti_osi_config_file: str) -> bool:
    """
    Verify the existence of the verti_osi.yaml file in the provided directory.

    Args:
        file_dir (str): The directory to check for the verti_osi.yaml file.
        verti_osi_config_file (str): The name of the verti_osi.yaml file.

    Returns:
        bool: True if the file exists.

    Raises:
        FileNotFoundError: If the directory or config file does not exist.
    """
    if not file_dir:
        raise FileNotFoundError("Directory path is empty.")

    if not os.path.isdir(file_dir):
        raise FileNotFoundError(f"Directory does not exist: {file_dir}")

    verti_osi_config_path = os.path.join(file_dir, verti_osi_config_file)
    if not os.path.isfile(verti_osi_config_path):
        raise FileNotFoundError(
            f"Config file does not exist: {verti_osi_config_path}")

    return True



import typer
import os
from vertibit_osi_image_generator.main.config_file_processor import load_and_validate_config
from vertibit_osi_image_generator.main.docker_file_generation import main_docker_file_generator
from vertibit_osi_image_generator.shared_aux.config_aux import format_image_params_to_schema, format_dockerfile_params_to_schema
from vertibit_osi_image_generator.shared_aux.language_package_managers import extract_images
from vertibit_osi_image_generator.shared_aux.language_package_manager_scanner import main_language_scanner
from vertibit_osi_image_generator.image_build.generate_docker_image import build_docker_image
from vertibit_osi_image_generator.shared_aux.shared import copy_file, generate_random_string
from vertibit_osi_image_generator.testing.checkov import checkov_check
from vertibit_osi_image_generator.verti_file_config_aux.file_config import verify_config_file
from typing import List
from importlib.resources import files

app = typer.Typer()


@app.command()
def image(
    file_dir: str = typer.Option(
        "", help="Path to the verti-osi.yaml config file. e.g: /dir/path"),
    platforms: str = typer.Option(
        "", help="Platforms the built image should support. e.g: linux/amd64,linux/arm64"),
    repository_branch: str = typer.Option(
        "", help="Repository branch to be used. e.g. main, develop"),
    repository_url: str = typer.Option(
        "", help="Remote repository URL. e.g. https://github.com/owner/repo"),
    env_vars: List[str] = typer.Option(
        [], help="Application ENV vars. e.g. KEY=VALUE,KEY=VALUE"),
    env_vars_rt: List[str] = typer.Option(
        [], help="Runtime ENV vars. e.g. KEY=VALUE,KEY=VALUE"),
    env_vars_bt: List[str] = typer.Option(
        [], help="Build-time ENV vars. e.g. KEY=VALUE,KEY=VALUE"),
    build_commands: List[str] = typer.Option(
        [], help="Build commands to run. e.g. npm run build, npm run lint"),
    pre_build_commands: List[str] = typer.Option(
        [], help="Pre-build commands to run before the build process."),
    root_directory: str = typer.Option('.', help="Project's root directory."),
    source_directory: str = typer.Option('.', help="Source code directory."),
    image_name: str = typer.Option(
        "", help="Name for the generated container image."),
    build_image: str = typer.Option(
        "", help="The build image to use."),
    run_time_image: str = typer.Option(
        "", help="The runtime image to use."),
    daemon: str = typer.Option(
        'docker', help="Container daemon to use (e.g., docker, podman)."),
    output: str = typer.Option(
        "", help="Generated image output type. Options: tar, registry, standard"),
    delete_generated_dockerfile: bool = typer.Option(
        False, help="Delete the generated Dockerfile after build."),
    run_generated_image: bool = typer.Option(
        False, help="Run the generated Docker image after build."),
    port: int = typer.Option(
        8080, help="The port to be exposed.")
):
    """
    Generate a container image.
    """
    # Define the config file name
    verti_osi_config_file = 'verti-osi.yaml'
    # Config file object to be referenced
    config = {}

    config_file = 'config.json'

    # Check if the config file is used to provided the parameters
    if file_dir != "":
        config_file_exists = verify_config_file(
            file_dir, verti_osi_config_file)
        # Check if the config file exists
        if config_file_exists:
            config_path = os.path.join(file_dir, verti_osi_config_file)
            # Validate and load the config file
            config = load_and_validate_config(
                image_config_path=config_path, config_file=config_file)

    else:
        # Format the cli parameters to a schema
        config = format_image_params_to_schema(platforms=platforms, repository_branch=repository_branch, repository_url=repository_url, env_vars=env_vars,
                                               env_vars_bt=env_vars_bt, env_vars_rt=env_vars_rt, build_commands=build_commands, pre_build_commands=pre_build_commands, root_directory=root_directory, source_directory=source_directory, image_name=image_name, daemon=daemon, output=output, delete_generated_dockerfile=delete_generated_dockerfile, port=port)

    # Scan the directory for the language package manager
    language_info = main_language_scanner(root_directory)

    # Generate a random image name if not provided
    if config['image-name'] == "":
        random_key = generate_random_string()
        image_name_prefix = 'verti-osi'

        config['image-name'] = f'{image_name_prefix}-{language_info['language'].lower()}-{random_key}'

    # Start directly with default images
    images = extract_images(language_info["language"])

    # Override build image if provided through parameters or config
    if build_image:
        images['build'] = build_image
    elif config.get('images', {}).get('build'):
        images['build'] = config['images']['build']

    # Override runtime image if provided through parameters or config
    if run_time_image:
        images['run-time'] = run_time_image
    elif config.get('images', {}).get('run-time'):
        images['run-time'] = config['images']['run-time']

    # Generating docker file content
    docker_file_content = main_docker_file_generator(
        language_info=language_info, images=images, source_directory=config['source-directory'], root_directory=config['root-directory'], env_vars=config['env-vars'], port=config['port'])

    # typer.echo(f"Identified language: {language_info['language']}")
    # typer.echo(f"Identified language images: {images}")

    # Ensure the 'tmp' directory exists
    os.makedirs("tmp", exist_ok=True)

    # Write the generated Dockerfile to a file
    with open("tmp/Dockerfile", "w") as f:
        f.write(docker_file_content)

    checkov_check("tmp/Dockerfile")

    if language_info["language"] == "NodeJs":
        docker_ignore_file_path = files("vertibit_osi_image_generator").joinpath(
            "config/docker-ignore/nodejs/.dockerignore")
        copy_file(source_path=docker_ignore_file_path,
                  destination_dir=config['root-directory'])
    if language_info["language"] == "Python":
        docker_ignore_file_path = files("vertibit_osi_image_generator").joinpath(
            "config/docker-ignore/python/.dockerignore")
        copy_file(source_path=docker_ignore_file_path,
                  destination_dir=config['root-directory'])

    build_docker_image(daemon=daemon, image_name=config['image-name'], container_file='tmp/Dockerfile',
                       build_context=config['root-directory'], output=config['output-type'], delete_generated_dockerfile=config['delete-generated-dockerfile'], run_generated_image=run_generated_image)


@app.command()
def dockerfile(
    file_dir: str = typer.Option(
        "", help="Path to the verti-osi.yaml config file. e.g: /dir/path"),
    repository_branch: str = typer.Option(
        "", help="Repository branch to be used. e.g. main, develop"),
    repository_url: str = typer.Option(
        "", help="Remote repository URL. e.g. https://github.com/owner/repo"),
    env_vars: List[str] = typer.Option(
        [], help="Application ENV vars. e.g. KEY=VALUE,KEY=VALUE"),
    env_vars_rt: List[str] = typer.Option(
        [], help="Runtime ENV vars. e.g. KEY=VALUE,KEY=VALUE"),
    env_vars_bt: List[str] = typer.Option(
        [], help="Build-time ENV vars. e.g. KEY=VALUE,KEY=VALUE"),
    build_commands: List[str] = typer.Option(
        [], help="Build commands to run. e.g. npm run build, npm run lint"),
    pre_build_commands: List[str] = typer.Option(
        [], help="Pre-build commands to run before the build process."),
    root_directory: str = typer.Option('.', help="Project's root directory."),
    source_directory: str = typer.Option('.', help="Source code directory."),
    output_directory: str = typer.Option(
        './', help="The output directory where the Dockerfile will be created."),
    build_image: str = typer.Option(
        "", help="The build image to use."),
    run_time_image: str = typer.Option(
        "", help="The runtime image to use."),
    port: int = typer.Option(
        8080, help="The port to be exposed.")
):
    """
    Generate a dockerfile.
    """
    # Define the config file name
    verti_osi_config_file = 'verti-osi.yaml'
    # Config file object to be referenced
    config = {}

    config_file = 'config.json'

    # Check if the config file is used to provided the parameters
    if file_dir != "":
        config_file_exists = verify_config_file(
            file_dir, verti_osi_config_file)
        # Check if the config file exists
        if config_file_exists:
            config_path = os.path.join(file_dir, verti_osi_config_file)
            # Validate and load the config file
            config = load_and_validate_config(
                image_config_path=config_path, config_file=config_file)

    else:
        # Format the cli parameters to a schema
        config = format_dockerfile_params_to_schema(repository_branch=repository_branch, repository_url=repository_url, env_vars=env_vars,
                                                    env_vars_bt=env_vars_bt, env_vars_rt=env_vars_rt, build_commands=build_commands, pre_build_commands=pre_build_commands, root_directory=root_directory, source_directory=source_directory, port=port)

    # Scan the directory for the language package manager
    language_info = main_language_scanner(root_directory)

    # Start directly with default images
    images = extract_images(language_info["language"])

    # Override build image if provided through parameters or config
    if build_image:
        images['build'] = build_image
    elif config.get('images', {}).get('build'):
        images['build'] = config['images']['build']

    # Override runtime image if provided through parameters or config
    if run_time_image:
        images['run-time'] = run_time_image
    elif config.get('images', {}).get('run-time'):
        images['run-time'] = config['images']['run-time']

    # Generating docker file content
    docker_file_content = main_docker_file_generator(
        language_info=language_info, images=images, source_directory=config['source-directory'], root_directory=config['root-directory'], env_vars=config['env-vars'], port=config['port'])

    # typer.echo(f"Identified language: {language_info['language']}")
    # typer.echo(f"Identified language images: {images}")

    # Ensure the 'tmp' directory exists
    os.makedirs("tmp", exist_ok=True)

    if not output_directory.endswith('/'):
        output_directory += '/'

    dockerfile_output_path = os.path.join(output_directory, "Dockerfile")
    # Write the generated Dockerfile to a file
    with open(dockerfile_output_path, "w") as f:
        f.write(docker_file_content)

    checkov_check(dockerfile_output_path)


if __name__ == "__main__":
    app()



import typer
from vertibit_osi_image_generator.shared_aux.shared import delete_files
from vertibit_osi_image_generator.shared_aux.extract_and_run_image import run_docker_image
from vertibit_osi_image_generator.shared_aux.comand_runner import run_and_log_commands


def stream_subprocess_output(process, queue, stream_name):
    """
    Read from a subprocess stream (stdout/stderr) line by line and put into a queue
    """
    stream = process.stdout if stream_name == 'stdout' else process.stderr
    for line in iter(stream.readline, ''):
        queue.put((stream_name, line.strip()))
    stream.close()


def build_docker_image(daemon, image_name, container_file, build_context, run_generated_image="False", delete_generated_dockerfile="False", output="", platforms: list = ["linux/amd64", "linux/arm64"]):
    """
    Builds a Docker image using the provided parameters.

    Args:
        daemon (str): Docker daemon to use (e.g., "docker" or "podman").
        image_name (str): Name of the Docker image to build.
        container_file (str): Path to the Dockerfile or Containerfile.
        build_context (str): Path to the build context.

    Raises:
        typer.Exit: Exits the CLI with an error code if the command fails.
    """
    formatted_platforms = ','.join(platforms)
    platform = f"--platform {formatted_platforms}"
    if output == 'push':
        command = f"{
            daemon} build {platform} -t {image_name} -f {container_file} -o type=registry,name={image_name} {build_context}"

    if output == 'tar':
        command = f"{
            daemon} build {platform} -t {image_name} -f {container_file} {build_context} && {daemon} save {image_name} -o {image_name}.tar"

    if output == 'normal' or output == '':
        command = f"{
            daemon} build {platform} -t {image_name} -f {container_file} {build_context}"

    try:
        run_and_log_commands(command)

        typer.echo(f"Docker image '{image_name}' built successfully!")

        if delete_generated_dockerfile == True:
            delete_files([container_file])

        if run_generated_image == True and output in ['push', 'normal', '']:
            run_docker_image(image_name)

        # if run_generated_image.lower() == 'true' and output == 'tar':
        #     run_docker_image_from_tgz(image_name, f'./tmp/{image_name}.tar')

    except Exception as e:
        typer.echo(f"An unexpected error occurred: {e}", err=True)
        raise typer.Exit(1)



from setuptools import setup, find_packages

setup(
    name="verti-osi",
    version="2.4.0",
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    author="VTB Wanderer DG",
    author_email="vtb.wanderers63@gmail.com",
    description="A simple Python package",
    url="https://github.com/vtb-wanderers63/py-logging-module",
    # Add other dependencies as needed
    install_requires=["typer", "pyyaml", "jsonschema"],
    entry_points={
        "console_scripts": [
            "verti-osi=vertibit_osi_image_generator.cli:app",  # Corrected entry point
        ],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
    include_package_data=True,
    package_data={
        "vertibit_osi_image_generator": ["config/config.json", "config/docker-ignore/nodejs/.dockerignore", "config/docker-ignore/python/.dockerignore"],
    }
)



from vertibit_osi_image_generator.nodejs.docker_file_generation import generate_nodejs_dockerfile
from vertibit_osi_image_generator.python.docker_file_generation import generate_python_dockerfile
from vertibit_osi_image_generator.nodejs.docker_file_generation_prep import prepeare_nodejs_docker_file_generation
from vertibit_osi_image_generator.python.docker_file_generation_prep import prepare_python_docker_file_generation


def main_docker_file_generator(language_info, images, source_directory, root_directory, env_vars, port):
    if language_info['language'] == 'NodeJs':
        nodejs_prep = prepeare_nodejs_docker_file_generation(
            root_directory=root_directory, source_directory=source_directory, language_info=language_info)
        package_directories = nodejs_prep['package_directories']
        installation_command = nodejs_prep['install_command']
        start_command = nodejs_prep['entry_point']
        return generate_nodejs_dockerfile(dev_base_image=images['build'], prod_base_image=images['run-time'], package_manager_dir=package_directories,
                                          entry_point=start_command, install_command=installation_command, source_dir=source_directory, environment_variables=env_vars, port=port)
    elif language_info['language'] == 'Python':
        python_prep = prepare_python_docker_file_generation(
            root_directory=root_directory, source_directory=source_directory, language_info=language_info)
        start_command = python_prep['entry_point']
        return generate_python_dockerfile(dev_base_image=images['build'], prod_base_image=images['run-time'], package_manager_dir=language_info['package_dir_path'],
                                          entry_point=start_command, source_dir=source_directory, environment_variables=env_vars, port=port)
    elif language_info['language'] == 'Maven':
        print("Maven dockerfile generation doesn't exist.")
        pass
    elif language_info['language'] == 'Gradle':
        print("Gradle dockerfile generation doesn't exist.")
        pass



from vertibit_osi_image_generator.shared_aux.shared import format_directory


def generate_python_dockerfile(
    dev_base_image: str,
    prod_base_image: str,
    package_manager_dir: str = None,
    entry_point: list = None,
    base_dir: str = "/base",
    runtime_dir: str = "/app",
    source_dir: str = "src",
    dependencies: str = '-r requirements.txt',
    environment_variables: list = [
        {"name": "PORT", "value": "8000", "type": "runtime"}],
    group_id: int = 1001,
    user_id: int = 1001,
    port: int = 8000
) -> str:
    """
    Generates a Dockerfile as a string based on the provided parameters.

    Parameters:
        base_image (str): The base image for the Dockerfile (e.g., "python:3.9-slim").
        working_dir (str): The working directory inside the container (default: "/app").
        environment_vars (dict): A dictionary of environment variables (default: None).
        install_commands (list): A list of shell commands to install dependencies (default: None).
        copy_commands (list): A list of (source, destination) tuples for files to copy (default: None).
        entrypoint (list): A list of entrypoint commands (default: None).

    Returns:
        str: A string representation of the generated Dockerfile.
    """
    lines = []

    # Development stage
    lines.append(f"FROM {dev_base_image} AS build-time")
    lines.append(f"WORKDIR {base_dir}")
    lines.append(f"COPY {package_manager_dir} {format_directory('.')}")

    lines.append(f"RUN python -m venv venv")
    lines.append(f"ENV PATH='{base_dir}/venv/bin:$PATH'")

    # Set required env variables
    if len(environment_variables) > 0:
        for env_var in environment_variables:
            if env_var['type'] == "both" or env_var['type'] == "buildtime":
                lines.append(f"ENV {env_var['name']}={env_var['value']}")

    lines.append(f"RUN pip install --no-cache-dir {dependencies}")

    # Section Spearator
    lines.append(f"####")

    # Final stage
    lines.append(f"FROM {prod_base_image} AS run-time")
    lines.append(f"WORKDIR {runtime_dir}")

    # Creating new user and group
    group_name = "pyappgroup"
    user_name = "pyappuser"

    lines.append(f"ARG UID={user_id}")
    lines.append(f"ARG GID={group_id}")

    lines.append(f"RUN addgroup --gid $GID {group_name} && adduser --uid $UID --ingroup {
                 group_name} --system {user_name}")

    if len(environment_variables) > 0:
        # Set required env variables
        for env_var in environment_variables:
            if env_var['type'] == "both" or env_var['type'] == "runtime":
                lines.append(f"ENV {env_var['name']}={env_var['value']}")

    lines.append(
        f"COPY --from=build-time {base_dir}/venv ./venv")
    lines.append(f"ENV PATH='{runtime_dir}/venv/bin:$PATH'")

    lines.append(
        f"COPY --chown={user_name}:{group_name} {source_dir} {format_directory(source_dir)}")
    
    lines.append(f"RUN chown -R {user_name}:{group_name} {runtime_dir}")

    # Section Spearator
    lines.append(f"####")

    lines.append(f"ENV PORT={port}")

    lines.append(f"EXPOSE {port}")

    lines.append(f"USER {user_name}")

    # Section Spearator
    lines.append(f"####")

    # Entrypoint
    lines.append(f'ENTRYPOINT {entry_point}')

    return "\n".join(lines)
